@@ -1,24 +1,11 @@
-### ace.R  (2006-06-27)
+### ace.R (2006-10-03)
 ###
-###            Ancestral Character Estimation
+###     Ancestral Character Estimation
 ###
 ### Copyright 2005-2006 Emmanuel Paradis and Ben Bolker
 ###
-### This file is part of the `ape' library for R and related languages.
-### It is made available under the terms of the GNU General Public
-### License, version 2, or at your option, any later version,
-### incorporated herein by reference.
-###
-### This program is distributed in the hope that it will be
-### useful, but WITHOUT ANY WARRANTY; without even the implied
-### warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-### PURPOSE.  See the GNU General Public License for more
-### details.
-###
-### You should have received a copy of the GNU General Public
-### License along with this program; if not, write to the Free
-### Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-### MA 02111-1307, USA
+### This file is part of the R-package `ape'.
+### See the file ../COPYING for licensing issues.
 
 ace <- function(x, phy, type = "continuous", method = "ML", CI = TRUE,
                 model = if (type == "continuous") "BM" else "ER",
@@ -27,14 +14,14 @@ ace <- function(x, phy, type = "continuous", method = "ML", CI = TRUE,
     if (class(phy) != "phylo")
       stop('object "phy" is not of class "phylo".')
     type <- match.arg(type, c("continuous", "discrete"))
-    tmp <- as.numeric(phy$edge)
-    nb.tip <- max(tmp)
-    nb.node <- -min(tmp)
-    if (nb.node != nb.tip - 1)      stop('"phy" is not rooted AND fully dichotomous.')
+    nb.tip <- length(phy$tip.label)
+    nb.node <- phy$Nnode
+    if (nb.node != nb.tip - 1)
+      stop('"phy" is not rooted AND fully dichotomous.')
     if (length(x) != nb.tip)
       stop("length of phenotypic and of phylogenetic data do not match.")
     if (!is.null(names(x))) {
-        if(!any(is.na(match(names(x), phy$tip.label))))
+        if(all(names(x) %in% phy$tip.label))
           x <- x[phy$tip.label]
         else warning('the names of argument "x" and the names of the tip labels
 did not match: the former were ignored in the analysis.')
@@ -45,43 +32,21 @@ did not match: the former were ignored in the analysis.')
         if (method == "pic") {
             if (model != "BM")
               stop('the "pic" method can be used only with model = "BM".')
-            phenotype <- as.numeric(rep(NA, nb.tip + nb.node))
-            names(phenotype) <- as.character(c(1:nb.tip, -(1:nb.node)))
-            if (is.null(names(x))) phenotype[1:nb.tip] <- x
-            else for (i in 1:nb.tip) phenotype[i] <- x[phy$tip.label[i]]
-            bl <- phy$edge.length
-            unused <- rep(TRUE, nb.tip + nb.node)
-            names(unused) <- names(phenotype)
-            contr <- numeric(nb.node)
-            names(contr) <- as.character(-(1:nb.node))
-            if (CI) var.con <- contr
-            while (sum(unused) > 1) {
-                term <- names(phenotype[!is.na(phenotype) & unused])
-                ind <- as.logical(match(phy$edge[, 2], term))
-                ind[is.na(ind)] <- FALSE
-                term.br <- matrix(phy$edge[ind], length(term), 2)
-                basal <- names(which(table(term.br[, 1]) == 2))
-                for (nod in basal) {
-                    pair.ind <- which(phy$edge[, 1] == nod)
-                    i <- pair.ind[1]
-                    j <- pair.ind[2]
-                    pair <- phy$edge[pair.ind, 2]
-                    a <- pair[1]
-                    b <- pair[2]
-                    if (scaled)
-                      contr[nod] <- (phenotype[a] - phenotype[b])/sqrt(bl[i] + bl[j])
-                    else contr[nod] <- phenotype[a] - phenotype[b]
-                    if (CI) var.con[nod] <- bl[i] + bl[j]
-                    unused[pair] <- FALSE
-                    phenotype[nod] <-
-                      (phenotype[a] * bl[j] + phenotype[b] * bl[i])/(bl[i] + bl[j])
-                    k <- which(phy$edge[, 2] == nod)
-                    bl[k] <- bl[k] + bl[i] * bl[j]/(bl[i] + bl[j])
-                }
-            }
-            obj$ace <- phenotype[-(1:nb.tip)]
-            se <- sqrt(var.con)
+            ## See pic.R for some annotations.
+            phy <- reorder(phy, "pruningwise")
+            phenotype <- numeric(nb.tip + nb.node)
+            phenotype[1:nb.tip] <- if (is.null(names(x))) x else x[phy$tip.label]
+            contr <- var.con <- numeric(nb.node)
+            ans <- .C("pic", as.integer(nb.tip), as.integer(nb.node),
+                      as.integer(phy$edge[, 1]), as.integer(phy$edge[, 2]),
+                      as.double(phy$edge.length), as.double(phenotype),
+                      as.double(contr), as.double(var.con),
+                      as.integer(CI), as.integer(scaled),
+                      PACKAGE = "ape")
+            obj$ace <- ans[[6]][-(1:nb.tip)]
+            names(obj$ace) <- (nb.tip + 1):(nb.tip + nb.node)
             if (CI) {
+                se <- sqrt(ans[[8]])
                 CI95 <- matrix(NA, nb.node, 2)
                 CI95[, 1] <- obj$ace + se * qnorm(0.025)
                 CI95[, 2] <- obj$ace - se * qnorm(0.025)
@@ -90,11 +55,12 @@ did not match: the former were ignored in the analysis.')
         }
         if (method == "ML") {
             if (model == "BM") {
+                tip <- phy$edge[, 2] <= nb.tip
                 dev <- function(p) {
-                    X <- c(x, p[-1])
-                    names(X) <- as.character(c(1:nb.tip, -(1:nb.node)))
-                    x1 <- X[phy$edge[, 1]]
-                    x2 <- X[phy$edge[, 2]]
+                    x1 <- p[-1][phy$edge[, 1] - nb.tip]
+                    x2 <- numeric(length(x1))
+                    x2[tip] <- x[phy$edge[tip, 2]]
+                    x2[!tip] <- p[-1][phy$edge[!tip, 2] - nb.tip]
                     -2 * (-sum((x1 - x2)^2/phy$edge.length)/(2*p[1]) -
                           nb.node * log(p[1]))
                 }
@@ -102,6 +68,7 @@ did not match: the former were ignored in the analysis.')
                            p = c(1, rep(mean(x), nb.node)), hessian = TRUE)
                 obj$loglik <- -out$minimum / 2
                 obj$ace <- out$estimate[-1]
+                names(obj$ace) <- (nb.tip + 1):(nb.tip + nb.node)
                 se <- sqrt(diag(solve(out$hessian)))
                 obj$sigma2 <- c(out$estimate[1], se[1])
                 se <- se[-1]
@@ -125,18 +92,18 @@ did not match: the former were ignored in the analysis.')
             if (class(corStruct)[1] %in% c("corBrownian", "corGrafen")) {
                 dis <- cophenetic.phylo(attr(corStruct, "tree"), full = TRUE)
                 MRCA <- mrca(attr(corStruct, "tree"), full = TRUE)
-                M <- dis["-1", MRCA]
+                M <- dis[as.character(nb.tip + 1), MRCA]
                 dim(M) <- rep(sqrt(length(M)), 2)
             }
             varAY <- M[-(1:nb.tip), 1:nb.tip]
             varA <- M[-(1:nb.tip), -(1:nb.tip)]
             V <- corMatrix(Initialize(corStruct, data.frame(x)),
                            corr = FALSE)
-            tmp <- solve(V)
-            obj$ace <- varAY %*% tmp %*% x
+            invV <- solve(V)
+            obj$ace <- varAY %*% invV %*% x
             if (CI) {
                 CI95 <- matrix(NA, nb.node, 2)
-                se <- sqrt((varA - varAY %*% tmp %*% t(varAY))[cbind(1:nb.node, 1:nb.node)])
+                se <- sqrt((varA - varAY %*% invV %*% t(varAY))[cbind(1:nb.node, 1:nb.node)])
                 CI95[, 1] <- obj$ace + se * qnorm(0.025)
                 CI95[, 2] <- obj$ace - se * qnorm(0.025)
                 obj$CI95 <- CI95
@@ -153,7 +120,7 @@ did not match: the former were ignored in the analysis.')
             rate <- matrix(NA, nl, nl)
             if (model == "ER") np <- rate[] <- 1
             if (model == "ARD") {
-                np <- nl * (nl - 1)
+                np <- nl*(nl - 1)
                 rate[col(rate) != row(rate)] <- 1:np
             }
             if (model == "SYM") {
@@ -163,58 +130,49 @@ did not match: the former were ignored in the analysis.')
                 rate[col(rate) < row(rate)] <- 1:np
             }
         } else {
+            if (ncol(model) != nrow(model))
+              stop("the matrix given as `model' is not square")
+            if (ncol(model) != nl)
+              stop("the matrix `model' must have as many rows
+as the number of categories in `x'")
             rate <- model
             np <- max(rate)
         }
+        index.matrix <- rate
+        index.matrix[cbind(1:nl, 1:nl)] <- NA
         rate[cbind(1:nl, 1:nl)] <- 0
-        rate[rate == 0] <- np + 1
+        rate[rate == 0] <- np + 1 # to avoid 0's since we will use this an numeric indexing
 
         liks <- matrix(0, nb.tip + nb.node, nl)
-        rownames(liks) <- as.character(c(1:nb.tip, -(1:nb.node)))
         for (i in 1:nb.tip) liks[i, x[i]] <- 1
+        phy <- reorder(phy, "pruningwise")
 
         Q <- matrix(0, nl, nl)
         dev <- function(p, output.liks = FALSE) {
             Q[] <- c(p, 0)[rate]
             diag(Q) <- -rowSums(Q)
-            unused <- rep(TRUE, nb.tip + nb.node)
-            done <- c(rep(TRUE, nb.tip), rep(FALSE, nb.node))
-            names(unused) <- names(done) <- rownames(liks)
-            while (sum(unused) > 1) {
-                term <- names(which(done & unused))
-                ind <- as.logical(match(phy$edge[, 2], term))
-                ind[is.na(ind)] <- FALSE
-                term.br <- matrix(phy$edge[ind], length(term), 2)
-                basal <- names(which(table(term.br[, 1]) == 2))
-                for (nod in basal) {
-                    pair.ind <- which(phy$edge[, 1] == nod)
-                    i <- pair.ind[1]
-                    j <- pair.ind[2]
-                    pair <- phy$edge[pair.ind, 2]
-                    a <- pair[1]
-                    b <- pair[2]
-                    tmp <- eigen(Q * phy$edge.length[i], symmetric = FALSE)
-                    P1 <- tmp$vectors %*% diag(exp(tmp$values)) %*% solve(tmp$vectors)
-                    tmp <- eigen(Q * phy$edge.length[j], symmetric = FALSE)
-                    P2 <- tmp$vectors %*% diag(exp(tmp$values)) %*% solve(tmp$vectors)
-                    liks[nod, ] <- P1 %*% liks[a, ] * P2 %*% liks[b, ]
-                    unused[pair] <- FALSE
-                }
-                done[basal] <- TRUE
+            for (i  in seq(from = 1, by = 2, length.out = nb.node)) {
+                j <- i + 1
+                anc <- phy$edge[i, 1]
+                des1 <- phy$edge[i, 2]
+                des2 <- phy$edge[j, 2]
+                tmp <- eigen(Q * phy$edge.length[i], symmetric = FALSE)
+                P1 <- tmp$vectors %*% diag(exp(tmp$values)) %*% solve(tmp$vectors)
+                tmp <- eigen(Q * phy$edge.length[j], symmetric = FALSE)
+                P2 <- tmp$vectors %*% diag(exp(tmp$values)) %*% solve(tmp$vectors)
+                liks[anc, ] <- P1 %*% liks[des1, ] * P2 %*% liks[des2, ]
             }
             if (output.liks) return(liks[-(1:nb.tip), ])
-            - 2 * log(sum(liks["-1", ]))
+            - 2 * log(sum(liks[nb.tip + 1, ]))
         }
-        out <- nlm(function(p) dev(p),
-                   p = rep(ip, length.out = np),
+        out <- nlm(function(p) dev(p), p = rep(ip, length.out = np),
                    hessian = TRUE)
         obj$loglik <- -out$minimum / 2
         obj$rates <- out$estimate
         if (any(out$gradient == 0))
-          warning("The likelihood gradient seems flat in at least one dimension (null gradient):\ncannot compute the standard-errors of the transition rates.\n")
+          warning("The likelihood gradient seems flat in at least one dimension (gradient null):\ncannot compute the standard-errors of the transition rates.\n")
         else obj$se <- sqrt(diag(solve(out$hessian)))
-        obj$index.matrix <- rate
-        diag(obj$index.matrix) <- NA
+        obj$index.matrix <- index.matrix
         if (CI) {
             lik.anc <- dev(obj$rates, TRUE)
             lik.anc <- lik.anc / rowSums(lik.anc)
